<?php
/*
	* 2018 - VEGA Informatique
	*
	* NOTICE OF LICENSE
	*
	* This source file is subject to the Open Software License (OSL 3.0)
	* that is bundled with this package in the file LICENSE.txt.
	* It is also available through the world-wide-web at this URL:
	* http://opensource.org/licenses/osl-3.0.php
	*
	* DISCLAIMER
	*
	* Do not edit or add to this file if you wish to upgrade PolarisLink to newer
	* versions in the future. 
	*
	*  @author VEGA Informatique <contact@vega-info.fr>
	*  @copyright  2018 VEGA Informatique
	*  @license    http://opensource.org/licenses/osl-3.0.php  Open Software License (OSL 3.0)
	*/

require_once(_PS_ROOT_DIR_ . "/modules/polarislink/classes/PolarisLinkImportBase.php");
require_once(_PS_ROOT_DIR_ . "/modules/polarislink/classes/PolarisLinkProductRef.php");

/*
	 * [NEW] Enregistre toutes les promos d'un coup POUR 1 PRODUIT (ne pas mélanger les produits)
	 * ex. : 		
		 <productdiscountplan skus="X,Y" eans="Z,ZZ">
                <discounts from="2018-11-23 00:00:00" to="2018-11-29 23:59:59" solde="0" type="percentage" tax="1">
                        <discount sku="50063307" ean="4054765246759">35.00</discount>
                        <discount sku="50063321" ean="4054765246773">35.00</discount>
                        <discount sku="50063338" ean="4054765246780">35.00</discount>
                        <discount sku="50063314" ean="4054765246766">35.00</discount>
                        <discount sku="50063291" ean="4054765246742">35.00</discount>
                </discounts>
                <discounts from="2018-11-30 08:00:00" to="2018-12-20 23:59:59" solde="1" type="percentage" tax="1">
                        <discount sku="50063307" ean="4054765246759">50.00</discount>
                        <discount sku="50063321" ean="4054765246773">50.00</discount>
                        <discount sku="50063338" ean="4054765246780">50.00</discount>
                        <discount sku="50063314" ean="4054765246766">50.00</discount>
                        <discount sku="50063291" ean="4054765246742">50.00</discount>
                </discounts>
        </productdiscountplan>
	 */
class PolarisLinkImport_productdiscountplan extends PolarisLinkImportBase
{
	/*
			Obtient la référence d'un XML
		*/
	protected function getReferenceFromData($data)
	{
		$ref = "";

		if (isset($data["skus"])) $ref .= " sku " . $data["skus"];

		return $ref;
	}

	protected function OnImportData($data)
	{
		if (!$this->Link->OPT_ALLOW_I_PROMOS)
			return;	// Pas d'importation de promotions !

		// On se positionne sur le bon shop !
		$this->SetPricesContext();

		// Détection du produit : collection des codes			
		$id_product = null;
		$discounts = array();

		// Localisation du produit en v9
		if (isset($data["skus"])) {
			$locator = $this->GetProductCombinationFor($data["skus"], isset($data["eans"]) ? $data["eans"] : null, false, true, true);
			$id_product = $locator["id_product"];
		}

		if (!$id_product)
			return; // produit non trouvé !

		// Chargement des références
		$skuResolver = new PolarisLinkProductRef($this->Link);
		$skuResolver->loadProduct($id_product);

		// Parcours des réductions            
		$typeTax = "0";
		$extensionSpecificPrice = [];
		foreach ($data->discounts as $discount) {

			$info = array(
				"from" => (string) $discount["from"],
				"to" => (string) $discount["to"],
				"solde" => $this->FirstNonEmpty((string) $discount["solde"], "0"),
				"name" => $this->FirstNonEmpty((string) $discount["name"], ""),
				"type" => $this->Link->OPT_PROMO_AMOUNT ? "amount" : "percentage",
				"tax" => $typeTax,
				"attrs" => array(),
			);

			foreach ($discount->discount as $d) {
				$id_product_attribute = $skuResolver->getProductAttributeIdFor((string) $d["sku"], isset($d["ean"]) ? (string) $d["ean"] : null);
				$v = round($info["type"] == "percentage" ? ((float) floatval((string) $d["perc"]) / (float) 100.0) : (float) floatval((string) $d["amount"]), 6);
				if (!empty($id_product_attribute) && $v != 0) {
					$info["attrs"][$id_product_attribute] = array(
						"id" => NULL,
						"o" => 0,
						"n" => $v,
						"is_sale_o" => false,
						"is_sale_n" => $info["solde"] == "1",
						"name_o" => '',
						"name_n" => truncString($info["name"], 96),
					);
				}
			}

			$info["slt"] = $info["from"] . '/' . $info["to"] . "/" . $info["type"] . "/" . $info["tax"];

			if (count($info["attrs"]) > 0)
				$discounts[] = $info;
		}

		// Chargement de toutes les combinaisons du produit ...
		$sql = "SELECT pa.id_product_attribute AS id FROM " . _DB_PREFIX_ . "product_attribute pa WHERE pa.id_product = " . (int)$id_product;
		foreach (Db::getInstance()->executeS($sql, true, false) as $row) {
			for ($i = 0; $i < count($discounts); $i++) {
				if (!isset($discounts[$i]["attrs"][$row["id"]]))
					$discounts[$i]["attrs"][(int) $row["id"]] = array("id" => NULL, "o" => 0, "n" => 0);
			}
		}

		// On parcours l'arbre pour savoir si on peut positionner une promo toute déclinaison confondue ...
		for ($i = 0; $i < count($discounts); $i++) {
			$reductionMinimum = null;

			foreach ($discounts[$i]["attrs"] as $pa) {
				if ($pa["n"] >= 0) {
					if (!$reductionMinimum || $reductionMinimum["n"] > $pa["n"])
						$reductionMinimum = $pa;
				}
			}

			$newRules = array();

			if ($reductionMinimum)
				$newRules[0] = array(
					"id" => NULL, "o" => 0, "n" => $reductionMinimum["n"], "is_sale_o" => false,
					"is_sale_n" => $reductionMinimum["is_sale_n"],
					"name_n" => $reductionMinimum["name_n"]
				);

			// Suppression des règles qui répète la règle principale !				
			foreach ($discounts[$i]["attrs"] as $id_attr => $rule) {
				if ($rule["n"] > 0 && $rule["n"] != $reductionMinimum["n"])
					$newRules[$id_attr] = $rule;
			}

			$discounts[$i]["attrs"] = $newRules;
		}

		// Chargeons les règles de prix spécifiques pour ce produit
		$recycle = array();

		$rules = Db::getInstance(_PS_USE_SQL_SLAVE_)->executeS('
						SELECT s.*, ps.is_sale as ps_is_sale, ps.name as ps_name
						FROM `' . _DB_PREFIX_ . 'specific_price` s
						LEFT JOIN `' . _DB_PREFIX_ . 'polarislink_spe_price` ps ON (ps.id_specific_price = s.id_specific_price)
						WHERE 
							s.id_product = ' . (int) $id_product . '
							AND s.id_specific_price_rule = 0
							AND s.id_currency = 0
							AND s.id_country = 0
							AND s.id_group = 0
							AND s.id_customer = 0
							AND s.from_quantity = 1
							AND s.id_cart = 0
						');

		// TODO : les promo saisies manuellement peuvent être identifiées facilement : elles n'existent pas dans polarislink_spe_price ^^

		foreach ($rules as $regle_existante) {
			if ($regle_existante["ps_is_sale"] !== null)
				$extensionSpecificPrice[$regle_existante["id_specific_price"]] = $regle_existante["id_specific_price"];

			$slt = $regle_existante["from"] . "/" . $regle_existante["to"] . "/" . $regle_existante["reduction_type"] . "/" . $regle_existante["reduction_tax"];
			$used = false;

			for ($i = 0; $i < count($discounts); $i++) {
				$discount = $discounts[$i];

				if ($slt == $discount["slt"]) {
					if (isset($discounts[$i]["attrs"][(int) $regle_existante["id_product_attribute"]])) {
						$used = true;

						$discounts[$i]["attrs"][(int) $regle_existante["id_product_attribute"]]["id"] = $regle_existante["id_specific_price"];
						$discounts[$i]["attrs"][(int) $regle_existante["id_product_attribute"]]["o"] = $regle_existante["reduction"];
						$discounts[$i]["attrs"][(int) $regle_existante["id_product_attribute"]]["is_sale_o"] = $regle_existante["ps_is_sale"] == true;
						$discounts[$i]["attrs"][(int) $regle_existante["id_product_attribute"]]["name_o"] = $regle_existante["ps_name"];
					}
				}
			}

			if (!$used)
				$recycle[] = $regle_existante["id_specific_price"];
		}

		$db = Db::getInstance();

		foreach ($discounts as $discount) {
			foreach ($discount["attrs"] as $id_attr => $d) {
				if ((empty($d["id"]) || $d["o"] != $d["n"])) {
					if (empty($d["id"])) {
						if (count($recycle) > 0)
							$spePrice = new SpecificPrice(array_shift($recycle));
						else
							$spePrice = new SpecificPrice();
					} else
						$spePrice = new SpecificPrice($d["id"]);

					$spePrice->id_product = $id_product;
					$spePrice->id_product_attribute = $id_attr;
					$spePrice->id_specific_price_rule = 0;
					$spePrice->id_shop = Shop::getContextShopID();
					$spePrice->id_shop_group = Shop::getContextShopGroupID();
					$spePrice->id_currency = 0;
					$spePrice->id_country = 0;
					$spePrice->id_group = 0;
					$spePrice->id_customer = 0;

					$spePrice->from = $this->GetDateTime($discount["from"]);
					$spePrice->to = $this->GetDateTime($discount["to"]);
					$spePrice->price = -1;
					$spePrice->from_quantity = 1;

					$spePrice->reduction = $d["n"];
					$spePrice->reduction_tax = empty($discount["tax"]) ? "0" : "1";
					$spePrice->reduction_type = $discount["type"];

					if (!$spePrice->save())
						throw new Exception("Erreur lors de l'enregistrement des prix promotionnels : " . $db->getMsgError());
					$d["id"] = $spePrice->id;
				}

				// Sauvegarde extension d'information !
				if (!isset($extensionSpecificPrice[$d["id"]])) {
					$query = sprintf('INSERT INTO `' . _DB_PREFIX_ . 'polarislink_spe_price` (id_specific_price) VALUES (%d)', $d["id"]);
					$db->query($query);
					$extensionSpecificPrice[$d["id"]] = $d["id"];
				}

				if ($d["is_sale_o"] != $d["is_sale_n"] || $d["name_o"] != $d["name_n"]) {
					$query = sprintf(
						'UPDATE `' . _DB_PREFIX_ . 'polarislink_spe_price` SET is_sale = %d, name = \'%s\' WHERE id_specific_price = %d',
						$d["is_sale_n"],
						pSQL(truncString($d["name_n"], 96)),
						$d["id"]
					);
					$db->query($query);
				}
			}
		}

		// Pour tous les prix spécifiques à recycler => poubelle !
		foreach ($recycle as $id) {
			$sp = new SpecificPrice($id);
			$sp->delete();
		}
	}
}
