<?php
	/*
	* 2018 - VEGA Informatique
	*
	* NOTICE OF LICENSE
	*
	* This source file is subject to the Open Software License (OSL 3.0)
	* that is bundled with this package in the file LICENSE.txt.
	* It is also available through the world-wide-web at this URL:
	* http://opensource.org/licenses/osl-3.0.php
	*
	* DISCLAIMER
	*
	* Do not edit or add to this file if you wish to upgrade PolarisLink to newer
	* versions in the future. 
	*
	*  @author VEGA Informatique <contact@vega-info.fr>
	*  @copyright  2018 VEGA Informatique
	*  @license    http://opensource.org/licenses/osl-3.0.php  Open Software License (OSL 3.0)
	*/

	/*
	    PolarisLinkJob représente un travail à faire se manifestant d'un fichier
		plat XML à importer, et d'une entrée dans la base de données
	*/
	require_once( _PS_ROOT_DIR_ . "/modules/polarislink/models/PolarisLinkJobs.php");	
	require_once("PolarisLinkCacheManager.php");
	require_once("PolarisLinkRefCache.php");
	require_once("PolarisLinkChecks.php");	
	
	class PolarisLinkJobManager
	{
		public $DebugMode = false;
		private $Link;
		private $RefsCache = null;
		private $TimeOutSecs = 600;

		private $Importers = array();		
		private $CurrentJobName = "";
		private $CacheManager = null;
		private $LangsCache = null;
		private $CountriesCache = null;
		private $TaxesCache = null;
		private $Taxes = null;		
		private $Currencies = null;
		
		public function __construct(PolarisLinkSite $link)
		{
			$this->Link = $link;
			$this->CacheManager = new PolarisLinkCacheManager();

			if ($link->OPT_REFS_CACHE)
				$this->RefsCache = new PolarisLinkRefCache($this->Link);
		}
		
		public function isUsingRefCache()
		{
			return $this->RefsCache != null;
		}
		
		/*
			Renvoi vrai si la taxe existe !
		*/
		public function isTaxesRulesExists($id_rule)
		{
			if ($this->Taxes == NULL)
			{
				$this->Taxes = array();
				foreach (TaxRulesGroup::getTaxRulesGroups() as $rule)
					$this->Taxes[$rule["id_tax_rules_group"]] = true;
			}
			
			return isset($this->Taxes[$id_rule]);
		}
		
		/*
			Retourne le groupe de taxe correspondant au taux et au pays renseigné
		*/
		public function getTaxesGroupFor($id_country, $tax)
		{
			$tax_val = sprintf("%.2f", floatval($tax));

			if ($this->TaxesCache == NULL)
				$this->TaxesCache = array();
			
			if (!isset($this->TaxesCache[$id_country]))
			{
				$existingTaxGroups = array();
				foreach (TaxRulesGroup::getTaxRulesGroups(true) as $group)
					$existingTaxGroups[$group["id_tax_rules_group"]] = true;

				$this->TaxesCache[$id_country] = array();
				foreach (TaxRulesGroup::getAssociatedTaxRatesByIdCountry($id_country) as $id_group => $rate)
					if (isset($existingTaxGroups[$id_group]))
						$this->TaxesCache[$id_country][sprintf("%.2f", floatval($rate))] = $id_group;
			}

			if (isset($this->TaxesCache[$id_country][$tax_val]))
				return $this->TaxesCache[$id_country][$tax_val];
						
			return NULL;
		}
		
		public function getCurrencyFor($iso, $symb, $name)
		{
			if ($this->Currencies == null)
			{
				$currencies = array("iso" => array(), "symb" => array(), "name" => array());
				foreach(Currency::getCurrencies() as $currency)
				{
					$id = $currency["id_currency"];
					$currencies["symb"][strtolower($currency["sign"])] = $id;
					$currencies["iso"][strtolower($currency["iso_code"])] = $id;
					$currencies["name"][strtolower($currency["name"])] = $id;
				}
				$this->Currencies = $currencies;
			}
			
			$search = strtolower(trim($iso));			
			if (!empty($search) && isset($this->Currencies["iso"][$search]))
				return $this->Currencies["iso"][$search];
			
			$search = strtolower(trim($symb));			
			if (!empty($search) && isset($this->Currencies["symb"][$search]))
				return $this->Currencies["symb"][$search];
			
			$search = strtolower(trim($name));			
			if (!empty($search) && isset($this->Currencies["name"][$search]))
				return $this->Currencies["name"][$search];
			
			return NULL;
		}
		
		public function getCountryForISO($isoCode)
		{			
			$isoCode = strtoupper($isoCode);
			if (!isset($this->CountriesCache[$isoCode]))
			{
				$this->CountriesCache[$isoCode] = Country::getByIso($isoCode);
			}
			
			return $this->CountriesCache[$isoCode];			
		}
		
		public function getLangFor($key)
		{
			if ($this->LangsCache == null)
			{
				$this->LangsCache = array();
				foreach (Language::getLanguages(false) as $lang)
				{
					$this->LangsCache["ps".$lang["id_lang"]] = $lang;
					$this->LangsCache[trim($lang["iso_code"])] = $lang;
					$this->LangsCache[trim($lang["language_code"])] = $lang;
					$this->LangsCache[trim(str_replace("-", "_", $lang["language_code"]))] = $lang;
				}
			}
			
			$key = strtolower($key);
			if (isset($this->LangsCache[$key]))
				return $this->LangsCache[$key];
			
			return NULL;
		}
		
		public function getArchivesPath()
		{			
			$path = _PS_ROOT_DIR_."/polarislink/jobs/archives/lk-".$this->Link->LinkID;	
			// On s'assure que le répertoire final existe
			if (!file_exists($path))
				if (!@mkdir($path, 0700, true))
					throw new Exception(sprintf("impossible de créer le répertoire des travaux %s", $path));
			// Et qu'il est protégé !!
			if (!file_exists(_PS_ROOT_DIR_."/polarislink/.htaccess"))
				file_put_contents(_PS_ROOT_DIR_."/polarislink/.htaccess", "# Apache 2.2\n<IfModule !mod_authz_core.c>\n\tOrder deny,allow\n\tDeny from all\n</IfModule>\n\n\n# Apache 2.4\n<IfModule mod_authz_core.c>\n\tRequire all denied\n</IfModule>\n");

			return $path;
		}
		
		public function getQueuePath()
		{
			$path = _PS_ROOT_DIR_."/polarislink/jobs/queue/lk-".$this->Link->LinkID;	
			// On s'assure que le répertoire final existe
			if (!file_exists($path))
				if (!@mkdir($path, 0700, true))
					throw new Exception(sprintf("impossible de créer le répertoire des travaux %s", $path));
			return $path;
		}
		
		public function setCache(PolarisLinkCache $cache)
		{
			return $this->CacheManager->add($cache);
		}
		
		public function getCache($type)
		{
			return $this->CacheManager->get($type);
		}
		
		/*
			Obtient une instance de la classe d'importation, avec un système de cache pour éviter
			d'avoir à le faire à chaque ligne
		*/
		public function getImporter($name)
		{
			if (array_key_exists($name, $this->Importers))
				return $this->Importers[$name];
				
			$module = NULL;
			if (preg_match("/\A([\w]+)\z/", $name) > 0)
			{
				$module_file = _PS_ROOT_DIR_."/modules/polarislink/classes/PolarisLinkImport_".$name.".php";
				if (file_exists($module_file))
				{
					require_once($module_file);

					$classname = "PolarisLinkImport_".$name;
					if (class_exists($classname))
					{
						$module = new $classname();
						$module->DebugMode = $this->DebugMode;
						$module->Init($this, $this->Link, $this->RefsCache);
					}
				}
			}
			
			// Cache
			$this->Importers[$name] = $module;
			
			return $module;
		}
		
		/*
			Obtient les informations de progression générale
		*/
		public static function getGlobalProgress($linkID = null)
		{
			$lastSyncStr = Configuration::get('VEGA_POLARIS_LINK_LAST_SYNC');
			$results = array(
							"working" => false,
							"done" => 0,
							"todo" => 0,
							"err" => Configuration::Get("VEGA_POLARIS_LINK_LAST_ERRS"),
							"last_sync" => empty($lastSyncStr) ? null : DateTime::createFromFormat('Y-m-d H:i:s', $lastSyncStr)->format("c")
						);

			$db = Db::getInstance();
			$rows = $db->executeS("SELECT lockname FROM "._DB_PREFIX_."polarislink_sites WHERE lockname <> '' ".(empty($linkID) ? "" : " AND id_link = ".intval($linkID)), true, false);
			if ($rows === false)
				throw new PrestaShopDatabaseException($db->getMsgError());

			$locks = array();
			foreach ($rows as $row)
				$locks[] = $row["lockname"];				
			
			if (self::checkIfLocksStillValid($locks))
			{
				$results["working"] = true;
				// Si on travaille, que les erreurs en cours
				$results["err"] = 0;
			}
									
			// Quoiqu'il arrive, on charge les travaux à effectuer ...
			$rows = $db->executeS("SELECT COALESCE(SUM(rows_done), 0) AS done, COALESCE(SUM(total_rows), 0) AS todo, COALESCE(SUM(err), 0) AS err FROM "._DB_PREFIX_."polarislink_jobs WHERE rows_done < total_rows ".(
					empty($linkID) ? "" : " AND id_link = ".intval($linkID)), true, false);
			if ($rows === false)
				throw new PrestaShopDatabaseException($db->getMsgError());
			if (count($rows) > 0)
			{
				$results["done"] += intval($rows[0]["done"]);
				$results["todo"] += intval($rows[0]["todo"]);
				$results["err"] += intval($rows[0]["err"]);
			}
			
			return $results;
		}
		
		public function testGZfile($filename)
		{
			$fp = fopen($filename, "r");
			$header = fread($fp, 2);
			fclose($fp);
			
			if (strlen($header) < 2) 
				return false;
			
			return (ord($header[0]) == 0x1f && ord($header[1]) == 0x8b);
		}
		
		private function openJobfile($jobFilename)
		{
			$wrapper = "file";
			
			if (!file_exists($jobFilename))
				throw new Exception(sprintf("le fichier de travail '%s' est introuvable !"));

			if ($this->testGZfile($jobFilename))
				$wrapper = "compress.zlib";
			
			$docxml = new XMLReader();
			if (!$docxml->open($wrapper."://".$jobFilename, 
											NULL, LIBXML_NOBLANKS | LIBXML_NOEMPTYTAG | LIBXML_NONET | LIBXML_PARSEHUGE))
							throw new Exception(sprintf("impossible d'ouvrir le fichier de travail (%s)", $jobFilename));
			
			$this->CurrentJobName = basename($jobFilename);
			
			return $docxml;
		}
		
		/*
			Ajoute un nouveau travail à la liste des choses à faire
		*/
		public function enqueueJob($filename)
		{
			if (!file_exists($filename))
				throw new Exception(sprintf("fichier %s introuvable", $filename));
			
			try
			{
				// Ok, on transfert le fichier dans notre queue de traitement						
				$queuePath = $this->getQueuePath();
				$finalFilename = $queuePath."/".sprintf("%d", microtime(true) * 10000000)."-".basename($filename);				
				
				// On déplace le fichier
				rename($filename, $finalFilename);
			}
			catch (Exception $ex)
			{
				// erreur
				if (file_exists($filename))
					unlink($filename);
				throw $ex;
			}
		}		

		/*
			Recherche les travaux à faire dans la queue de traitement
		*/
		public function scanJobs()
		{
			clearstatcache();
			
			// Ok, on transfert le fichier dans notre queue de traitement						
			$queuePath = $this->getQueuePath();						
			
			// Etape 1, on se constitue une db de ce qu'on a déjà répertorié
			$jobs = array();				

			$db = Db::getInstance();
			$rows = $db->executeS(
				"SELECT id_job, job, rows_done >= total_rows AS finished, err FROM "._DB_PREFIX_."polarislink_jobs WHERE id_link = ".intval($this->Link->LinkID)."", true, false);

			if ($rows === false)
				throw new PrestaShopDatabaseException($db->getMsgError());
			foreach ($rows as $row)
			{
				$jobs[$row['job']] = $row['id_job'];
				
				$jobFilename = $queuePath."/".$row['job'];
				if ($row["finished"] && file_exists($jobFilename))	// fini, on fait le ménage
				{
					if ($this->Link->OPT_USE_ARCHIVES || $row["err"] > 0)
						rename($jobFilename, $this->getArchivesPath()."/".$row['job']);
					else
						unlink($jobFilename);
				}
			}
			
			// Etape 2, on scanne le répertoire
			$excludeFiles = array( "index.html", "index.htm", "index.php" );
			clearstatcache();
			foreach (scandir($queuePath, SCANDIR_SORT_ASCENDING) as $filename)
			{
				if ($filename != "" && substr($filename, 0, 1) != "." && !in_array($filename, $excludeFiles))
				{
					if (isset($jobs[$filename]))
					{
						// On le retire de la liste
						unset($jobs[$filename]);
					}
					else
					{
						try
						{
							// Pb NFS : on attends que le fichier arrive ?
							$path = $queuePath."/".$filename;

							$tries = 3;
							while ($tries > 0)
							{
								clearstatcache();

								if (file_exists($path) && filesize($path) > 0)
									break;								

								sleep(10);
								$tries--;
							}

							clearstatcache();

							if (!file_exists($path))
								throw new Exception(sprintf("le fichier soumis (%s) est introuvable", $path));

							if (filesize($path) == 0)
								throw new Exception(sprintf("le fichier soumis (%s) est vide", $path));

							// On parcours le fichier
							$docxml = $this->openJobfile($path);
							try
							{
								// on parcours le fichier pour savoir ce qu'il y a faire
								$todo = 0;
								//$docxml->setParserProperty(XMLReader::VALIDATE, true);

								$emptyCtrl = 0;
								$depth = 0;	// Profondeur					
								while (@$docxml->read())
								{
									/*if (!$docxml->isValid())
										throw new Exception("le fichier soumis n'est pas un XML valide");*/
								
									$emptyCtrl++;
									switch ($docxml->nodeType)
									{
										case XMLReader::ELEMENT:
											$depth++;
											if ($depth == 1)
											{
												if ($docxml->name != "import")
													throw new Exception(sprintf("racine '%s' inconnue dans le document XML, 'import' attendu", $docxml->name));
											}
											else if ($depth == 2)
												$todo++;							
											
											if ($docxml->isEmptyElement)
												$depth--;
											break;
										case XMLReader::END_ELEMENT:
											$depth--;
											break;
									}
								}
								
								if ($emptyCtrl == 0)
								{
									throw new Exception(sprintf("le fichier soumis (%s) ne contient pas de balise xml", $path));
								}
								
								$docxml->close();
								
								// Insertion dans la base de données
								$job = new PolarisLinkJobs();
								$job->id_link = $this->Link->LinkID;
								$job->creation_date = time();
								$job->priority = 50;
								$job->rows_done = 0;
								$job->total_rows = $todo;
								$job->job = $filename;
								$job->save();
							}
							catch (Exception $ex)
							{
								$docxml->close();
								throw $ex;
							}
						}
						catch (Exception $ex)
						{
							// erreur
							if (file_exists($queuePath."/".$filename))
							{
								if ($this->Link->OPT_USE_ARCHIVES)
									rename($queuePath."/".$filename, $this->getArchivesPath()."/".$filename);
								else
									unlink($queuePath."/".$filename);
							}
							
							$this->logError($ex);
						}
					}
				}
			}
						
			foreach ($jobs as $job_id)
			{
				try
				{
					// purge 
					$job = new PolarisLinkJobs($job_id);
					$job->delete();
				}
				catch (Exception $ex)
				{
					// Ne rien faire
				}
			}
		}
		
		/*
			Enregistre une erreur
		*/
		public function logError($ex)
		{
			$error = "";
			
			if (is_a($ex, "Exception"))
			{
				$errors = array();
				while ($ex != null)
				{
					$msg = $ex->getMessage();
					
					if (true)
						$msg .= " @ ".basename($ex->getFile())." Lg. ".$ex->getLine();
					
					$errors[] = $msg;
					
					$ex = $ex->getPrevious();
				}
				
				$error = join($errors, " ➔ ");				
			}
			else
				$error = $ex;

			if (!empty($this->CurrentJobName))
				$errMsg = "Erreur import « ".$this->CurrentJobName." » : " . $error;
			else
				$errMsg = "Erreur import : " . $error;

			PolarisLink::Log(LOG_ERR, $errMsg, $this->Link->LinkID, null, null, true);
		}
		
		/*
		 * Si verrou exclusif, alors n'acquiert le verrou que si aucun
		 * autre lien ne l'a déjà acquis
		 */
		public static function AcquireLock($linkID, $exclusive = false)
		{
			$db = Db::getInstance();
			
			$rows = $db->executeS("SELECT lockname FROM "._DB_PREFIX_."polarislink_sites WHERE lockname <> ''". (!$exclusive ? " AND id_link = ".$linkID : ""), true, false);
			if ($rows === false)
				throw new PrestaShopDatabaseException($db->getMsgError());

			$locks = array();
			foreach ($rows as $row)			
				$locks[] = $row["lockname"];
				
			if (self::checkIfLocksStillValid($locks))
				return false;
			
			// Nous arrivons ici, c'est que tout est libre ...
			// on acquiert le verrou en deux passes ...
			$connId = $db->getValue("SELECT connection_id()");
			
			// On fixe le verrou,
			if (!$db->update("polarislink_sites", array("lockname" => $connId), 'id_link = '.$linkID, 0, false, false))
				throw new PrestaShopDatabaseException($db->getMsgError());
			
			// et on vérifie qu'un autre n'est pas déjà passé (le verrou positionné n'est pas le notre, malgré notre requête)
			$rows = $db->executeS("SELECT lockname FROM "._DB_PREFIX_."polarislink_sites WHERE id_link = ".$linkID, true, false);
			if ($rows === false)
				throw new PrestaShopDatabaseException($db->getMsgError());

			foreach ($rows as $row)			
				if ($row["lockname"] != $connId)
					return false;	// quelqu'un a pris la main !!
			
			// Ok, ça a l'air bon !
			return true;
		}
		
		public static function checkIfLocksStillValid($locks)
		{
			if (!empty($locks))
			{				
				if (!is_array($locks))
					$locks = array( $locks );
			
				$db = Db::getInstance();
				
				$connId = (string) $db->getValue("SELECT connection_id()");
				if (empty($connId))
					throw new Exception("Erreur à la vérification des verrous : impossible d'obtenir l'identifiant de la connexion !");

				if (in_array($connId, $locks))
					return true;	// C'est nous !
				
				if (!empty($locks))
				{
					// Il y a déjà un processus déclaré, 
					// on vérifie qu'il existe encore ...
					$foundMyself = false;
					$ps = $db->executeS("SHOW PROCESSLIST", true, false);
					if (!empty($ps))
					{
						foreach ($ps as $row)
						{		
							$id = (string) $row["Id"];			
							if (in_array($id, $locks))
								return true;	// Oui, le processus est toujours là !!
							if ($id == $connId)
								$foundMyself = true;
						}
					}

					// Toujours là ?
					// On s'assure que le SHOW PROCESSLIST fonctionne
					// Normalement, on devrait s'être trouvé nous même ??!
					if (!$foundMyself)
						throw new Exception("Erreur à la vérification des verrous : impossible d'obtenir la liste des connexions SQL actives (SHOW PROCESSLIST) !");
				}
			}
						
			// Tout est libre !
			return false;
		}
		
		public static function ReleaseLock($linkID)
		{
			$db = Db::getInstance();
			
			// On relache le verrou,
			if (!$db->update("polarislink_sites", array("lockname" => ""), 'id_link = '.$linkID, 0, false, false))
				throw new Exception($db->getMsgError());
		}
		
		/* 
			Execution d'un travail
		*/
		private function parseXml(XMLReader $docxml, $job)
		{
			$skipNodes = $job->rows_done;
			
			$db = Db::getInstance();
			$depth = 0;	// Profondeur				
			$nb_nodes = 0;	// Nombre de noeuds regardés
			$lastClass = null;	// Dernière classe de sérialisation utilisée
			$lastClassName = null;	// Nom de la dernière classe de sérialisation utilisée

			$lastTimeSaveJobStats = microtime( true );
			$lastTimeResetTimer = microtime( true );
			
			set_time_limit($this->TimeOutSecs);			
			while ($docxml->read())
			{
				switch ($docxml->nodeType)
				{
					case XMLReader::ELEMENT:
						$depth++;
						
						if ($depth == 1)
						{
							if ($docxml->name != "import")
								throw new Exception(sprintf("racine '%s' inconnue dans le document XML, 'import' attendu", $docxml->name));

							// Version & Licence ?
							$saveCfg = false;
							$attrLic = $docxml->getAttribute("pl_licence");
							if (!empty($attrLic) && $attrLic != $this->Link->PolarisLicence)
							{
								$this->Link->PolarisLicence = $attrLic;
								$saveCfg = true;
							}

							$attrVer = $docxml->getAttribute("pl_version");
							if (!empty($attrVer) && $attrVer != $this->Link->PolarisVersion)
							{
								$this->Link->PolarisVersion = $attrVer;
								$saveCfg = true;
							}

							if ($saveCfg)
								$this->Link->save();
						}
						else if ($depth == 2)
						{
							$nb_nodes++;
							while ($nb_nodes <= $skipNodes)
							{
								// On saute tous les noeuds déjà fait !
								switch ($docxml->nodeType)
								{
									case XMLReader::ELEMENT:										
										if ($docxml->isEmptyElement)
										{
											if ($depth == 2) $nb_nodes++;											
										}
										else											
											$depth++;
										break;
									case XMLReader::END_ELEMENT:
										$depth--;
										if ($depth == 2) $nb_nodes++;										
										break;
								}
								
								// Prochain noeud 
								if (!$docxml->read())
								{
									$job->rows_done = $nb_nodes;
									$job->save();
									return;	// fini !
								}
							}

							$now = microtime( true );
							if ( ($now - $lastTimeResetTimer) >= 10)
							{
								// On s'accorde un nouveau timeout !
								set_time_limit($this->TimeOutSecs);
								$lastTimeResetTimer = $now;
							}			

							try
							{
								$nclass = (string) $docxml->name;
								if ($lastClassName != $nclass)
								{
									// Pas la peine de réexecuter ceci, on garde en cache
									$lastClassName = $nclass;
									$lastClass = $this->GetImporter($nclass);
									if (!$lastClass)
										throw new Exception(sprintf("élément '%s' inconnu", $nclass));
								}
								if ($lastClass != null)
									$lastClass->import($docxml);

								// Marquage de ce qui a été fait
								$job->rows_done = $nb_nodes;
							}
							catch (PrestaShopDatabaseException $pex)
							{
								// Fin du travail ...
								$this->logError($pex);
								$job->save();
								// Fin de traitement
								die();
							}
							catch (Exception $ex)
							{
								// Marquage de là où nous en étions
								$job->err++;
								$job->rows_done = $nb_nodes;
																
								$this->logError($ex);
							}
														
							// Toutes les secondes
							$now = microtime( true ); 
							if ( ( $now - $lastTimeSaveJobStats) >= 1)
							{
								$job->save();
								$lastTimeSaveJobStats = $now;
							}
						}
						if ($docxml->isEmptyElement)
							$depth--;
						break;
					case XMLReader::END_ELEMENT:
						$depth--;
						break;
					default:
						break;
				}
			}			
						
			$job->save();
		}	

		/*
			Taches CRON
		*/
		private function doCronTasks($force = false)
		{
			// Nettoyage des tables de stats ?
			if (Configuration::get("POLARISLINK_AUTOCLEAN_STATS"))
			{
				$lastDone = Configuration::get("POLARISLINK_AUTOCLEAN_STATS_DONE");		
				if ($force || $lastDone < date("Y-m-d") )
				{
					Configuration::updateValue("POLARISLINK_AUTOCLEAN_STATS_DONE", date("Y-m-d"));

					// Nettoyage 
					$db = Db::getInstance();

					if (!$db->execute("DELETE FROM "._DB_PREFIX_."pagenotfound WHERE date_add < DATE_SUB(NOW(), INTERVAL 1 MONTH)"))
						throw new Exception($db->getMsgError());

					if (!$db->execute("DELETE FROM "._DB_PREFIX_."connections WHERE date_add < DATE_SUB(NOW(), INTERVAL 1 MONTH)"))
						throw new Exception($db->getMsgError());

					if (!$db->execute("DELETE FROM "._DB_PREFIX_."connections_page WHERE id_connections NOT IN (SELECT id_connections FROM "._DB_PREFIX_."connections)"))
						throw new Exception($db->getMsgError());
					
					if (!$db->execute("DELETE FROM "._DB_PREFIX_."guest WHERE id_guest NOT IN (SELECT id_guest FROM "._DB_PREFIX_."cart) AND id_guest NOT IN (SELECT id_guest FROM "._DB_PREFIX_."connections)"))
						throw new Exception($db->getMsgError());					
				}
			}
		}
		
		/* 
			Execute les travaux
		*/
		public function run()
		{
			// Surtout plus de cache !!!
			Db::getInstance()->disableCache();

			try			
			{
				if (!self::AcquireLock($this->Link->LinkID))
					return;	// Toujours en train de tourner
			}
			catch (Exception $ex)
			{
				$this->logError($ex);
				return;
			}

			try
			{
				// Timeout pour lire les XML et faire les tâches CRON
				set_time_limit($this->TimeOutSecs);
				
				// Reset du cache = moins de soucis
				if ((defined('_PS_CACHE_ENABLED_')) ? _PS_CACHE_ENABLED_ : false) 
				{
					if (version_compare(_PS_VERSION_, 1.7) >= 0)	// Version 1.7 uniquement
						Cache::clear();
					Cache::getInstance()->delete('*');
				}

				$this->doCronTasks();
				
				$hasJob = true;
				while ($hasJob)
				{
					// On scanne le répertoire des choses à faire
					$this->scanJobs();
				
					$todo = PolarisLinkJobs::loadAll($this->Link->LinkID);
					$hasJob = ($todo != null && count($todo) > 0);
					if (!$hasJob)
						break;
										
					foreach ($todo as $job)
					{
						// Timeout ouvrir le XML de modifications
						set_time_limit($this->TimeOutSecs);

						try
						{
							Configuration::updateValue("VEGA_POLARIS_LINK_LAST_SYNC", date("Y-m-d H:i:s", $job->creation_date));

							$docxml = $this->openJobfile($this->getQueuePath()."/".$job->job);
							try
							{								
								// Et on traite !
								$this->parseXml($docxml, $job);
								$docxml->close();
							}
							catch (Exception $ex)
							{
								$docxml->close();
								throw $ex;
							}
														
							Configuration::updateValue("VEGA_POLARIS_LINK_LAST_ERRS", Configuration::Get("VEGA_POLARIS_LINK_LAST_ERRS") + $job->err);
						}
						catch (Exception $ex)
						{
							$this->logError($ex);
						}
					}
				}
			}
			catch (Exception $ex)
			{
				$this->logError($ex);
			}
			
			self::ReleaseLock($this->Link->LinkID);
		}		
	}
	